<?php
/*
  $Id: rcm_sc_recently_viewed.php
  $Loc: catalog/includes/modules/content/shopping_cart/

  based on 2.0 2008-10-28 Kymation $

  Recently Viewed 5.3.4 Phoenix Cart
  by @raiwa
  info@oscaddons.com
  www.oscaddons.com

  Copyright (c) 2021: Rainer Schmied - @raiwa

  CE Phoenix, E-Commerce made Easy
  https://phoenixcart.org

  Copyright (c) 2021 Phoenix Cart

  Released under the GNU General Public License
*/

  class cm_sc_recently_viewed extends abstract_executable_module {

    const CONFIG_KEY_BASE = 'MODULE_CONTENT_SC_RECENTLY_VIEWED_';

    public function __construct() {
      parent::__construct(__FILE__);

      $this->description .= '<p>by @raiwa <u><a target="_blank" href="http://www.oscaddons.com">www.oscaddons.com</a></u></p>';

      if (!file_exists(DIR_FS_CATALOG . 'templates/' . TEMPLATE_SELECTION . '/includes/hooks/shop/siteWide/recentlyViewed.php')) {
        $this->description = '<div class="alert alert-danger">' . MODULE_CONTENT_SC_RECENTLY_VIEWED_ERROR_HOOK . '</div>' . $this->description;
        $this->enabled = false;
      }
    }

    function execute() {

      $recently_viewed = $_SESSION['recently_viewed'] ?? null;

      if (!empty($recently_viewed)) {
        $ids_in_cart_list = null;

        $products = $_SESSION['cart']->get_products();
        if ( $_SESSION['cart']->count_contents() !== 0 ) {
          foreach ($products as $product) {
            $ids_in_cart_array[] = $product->get('id');
          }
          $ids_in_cart_list = implode(',', $ids_in_cart_array);
        }

        if (MODULE_CONTENT_SC_RECENTLY_VIEWED_DISPLAY_ORDER == 'Newest') {
          $products_query_order = " FIELD(p.products_id," . $recently_viewed . ") ASC";
        } elseif (MODULE_CONTENT_SC_RECENTLY_VIEWED_DISPLAY_ORDER == 'Oldest') {
          $products_query_order = " FIELD(p.products_id," . $recently_viewed . ") DESC";
        } else {
          $products_query_order = " rand()";
        }

        $recently_products_sql = sprintf(<<<'EOSQL'
SELECT DISTINCT %s
EOSQL
        , Product::COLUMNS);
      
        if (MODULE_CONTENT_SC_RECENTLY_VIEWED_SHOW_DESCRIPTION == 'True') {
          $recently_products_sql .= sprintf(<<<'EOSQL'
, SUBSTRING_INDEX(pd.products_description, " ", %s) AS products_description
EOSQL
          , MODULE_CONTENT_SC_RECENTLY_VIEWED_DESCRIPTION_LENGTH);
        }

        $recently_products_sql .= <<<'EOSQL'
  FROM products p
    LEFT JOIN specials s ON p.products_id = s.products_id
    INNER JOIN products_description pd ON p.products_id = pd.products_id
    LEFT JOIN (SELECT products_id, COUNT(*) AS attribute_count FROM products_attributes GROUP BY products_id) a ON p.products_id = a.products_id
  WHERE p.products_status = 1
EOSQL
        ;
      
        if ( MODULE_CONTENT_SC_RECENTLY_VIEWED_SHOW_CURRENT == 'False' && !empty($ids_in_cart_list) ) {
          $recently_products_sql .= sprintf(<<<'EOSQL'
     AND p.products_id NOT IN (%s)
EOSQL
          , Text::input($ids_in_cart_list));
        }
        
        $recently_products_sql .= sprintf(<<<'EOSQL'
    AND p.products_id IN (%s)
    AND pd.language_id = %d
  ORDER BY %s 
  LIMIT %d
EOSQL
        , Text::input($recently_viewed),
          (int)$_SESSION['languages_id'], 
          $products_query_order,
          (int)MODULE_CONTENT_SC_RECENTLY_VIEWED_CONTENT_LIMIT);
          
        $recently_products_query = $GLOBALS['db']->query($recently_products_sql);
        if (mysqli_num_rows($recently_products_query) > 0) {
          $card = [
            'show_buttons' => true,
          ];

          $tpl_data = [ 'group' => $this->group, 'file' => __FILE__ ];
          include 'includes/modules/content/cm_template.php';
        }
      }
    }

    protected function get_parameters() {
      return [
        'MODULE_CONTENT_SC_RECENTLY_VIEWED_VERSION_INSTALLED' => [
          'title' => 'Current Version',
          'value' => '5.3.4',
          'desc' => 'Version info. It is read only',
          'set_func' => 'cm_sc_recently_viewed::readonly(',
        ],
        'MODULE_CONTENT_SC_RECENTLY_VIEWED_STATUS' => [
          'title' => 'Enable Continue shopping & Recently module',
          'value' => 'True',
          'desc' => 'Do you want to enable this module?',
          'set_func' => "Config::select_one(['True', 'False'], ",
        ],
        'MODULE_CONTENT_SC_RECENTLY_VIEWED_CONTENT_WIDTH' => [
          'title' => 'Content Width',
          'value' => '12',
          'desc' => 'What width container should the content be shown in? (12 = full width, 6 = half width).',
          'set_func' => "Config::select_one(['12', '11', '10', '9', '8', '7', '6', '5', '4', '3', '2', '1'], ",
        ],
        'MODULE_CONTENT_SC_RECENTLY_VIEWED_SORT_ORDER' => [
          'title' => 'Sort Order',
          'value' => '1200',
          'desc' => 'Sort order of display. Lowest is displayed first.',
        ],
        'MODULE_CONTENT_SC_RECENTLY_VIEWED_CONTENT_LIMIT' => [
          'title' => 'Number of Products',
          'value' => '4',
          'desc' => 'Maximum Number of products that should show in this module?',
        ],
        'MODULE_CONTENT_SC_RECENTLY_VIEWED_SHOW_DESCRIPTION' => [
          'title' => 'Show the product description',
          'value' => 'False',
          'desc' => 'Show the product description in the Recently Viewed module?',
          'set_func' => "Config::select_one(['True', 'False'], ",
        ],
        'MODULE_CONTENT_SC_RECENTLY_VIEWED_DESCRIPTION_LENGTH' => [
          'title' => 'Maximum description length',
          'value' => '25',
          'desc' => 'The number of words of the description to display in the recently viewed module',
        ],
        'MODULE_CONTENT_SC_RECENTLY_VIEWED_SHOW_MORE_BUTTON' => [
          'title' => 'Show the product MORE button',
          'value' => 'True',
          'desc' => 'Show the MORE button in this module?',
          'set_func' => "Config::select_one(['True', 'False'], ",
        ],
        'MODULE_CONTENT_SC_RECENTLY_VIEWED_SHOW_CURRENT' => [
          'title' => 'Show the current product',
          'value' => 'False',
          'desc' => 'Show the current product in the Recently Viewed module?',
          'set_func' => "Config::select_one(['True', 'False'], ",
        ],
        'MODULE_CONTENT_SC_RECENTLY_VIEWED_DISPLAY_ORDER' => [
          'title' => 'Product order',
          'value' => 'Newest',
          'desc' => 'Show the newest or oldest recently viewed product at the beginning of the Recently Viewed module? Or use random order.',
          'set_func' => "Config::select_one(['Newest', 'Oldest', 'Random'], ",
        ],
      ];
    }

    public static function readonly($value) {
      return $value;
    }

  }
